#!perl -T
use 5.006;
use strict;
use warnings FATAL => 'all';
use Test::More;

unless ( $ENV{RELEASE_TESTING} ) {
    plan( skip_all => "Author tests not required for installation" );
}

my $min_tcm = 0.9;
eval "use Test::CheckManifest $min_tcm";
plan skip_all => "Test::CheckManifest $min_tcm required" if $@;

ok_manifest(
    {
        'filter' => [
            qr/\.git/,
            qr/\/tmp\//,
            qr/\/inc\/Module\/AutoInstall/,
            qr/\/inc\/Module\/Install/,
            qr/\.swp/,
            qr/\/local_files\//,
            qr/\/\.travis\.yml/,
            qr/\/cover_db/,
            qr/\/Docker/,
        ],
    }
);
